import { BraintreePaymentDetails } from '../../models/braintree-payment-data.model';
import {EventEmitter, Injectable} from '@angular/core';
import { Occ, TranslationService, PaymentDetails } from '@spartacus/core';
import {combineLatest, Observable} from 'rxjs';
import {Card} from '@spartacus/storefront';
import {map} from 'rxjs/operators';
import {paymentText} from '../../models/braintree.constants';
import Address = Occ.Address;

@Injectable({
  providedIn: 'root'
})
export class BraintreePaymentMethodsUtilsService {

  protected savePaymentInfo = false;
  protected billingAddress: Address;
  protected _backToPaymentMethods: EventEmitter<any>;

  constructor(
    protected translation: TranslationService
  ) { }

  toggleSavePaymentInfo(): void {
    this.savePaymentInfo = !this.savePaymentInfo;
    console.log(this.savePaymentInfo);
  }

  setSavePaymentInfo(savePaymentInfo: boolean): void{
    this.savePaymentInfo = savePaymentInfo;
  }

  getSavePaymentInfo(): boolean{
    return this.savePaymentInfo;
  }

  setBillingAddress(address: Address): void{
    this.billingAddress = address;
  }

  getBillingAddress(): Address{
    return this.billingAddress;
  }

  getPaymentMethodCardBraintreeMyAccount(paymentDetails: PaymentDetails): Observable<Card> {
    return combineLatest([
      this.translation.translate('paymentCard.defaultPaymentMethod'),
      this.translation.translate('paymentForm.payment'),
      this.translation.translate('paymentCard.expires', {
        month: paymentDetails.expiryMonth,
        year: paymentDetails.expiryYear,
      }),
      this.translation.translate('paymentForm.billingAddress'),
    ]).pipe(
      map(([textDefaultPaymentMethod, textTitle, textExpires, billingAddress]) => {
        const textDefault = paymentDetails.defaultPayment ? textDefaultPaymentMethod : '';

        const region = paymentDetails.billingAddress?.region?.isocode
          ? paymentDetails.billingAddress?.region?.isocode + ', '
          : '';

        const myAccountActions: { name: string; event: string }[] = [];
        if (!paymentDetails.defaultPayment) {
          myAccountActions.push({ name: 'Set as default', event: 'default' });
        }
        myAccountActions.push({ name: 'Delete', event: 'edit' });

        return {
          header: textDefault,
          title: textTitle,
          textBold: paymentDetails.subscriptionId,
          text: this.configurePaymentMethodMyAccountCardText(paymentDetails, textExpires),
          paragraphs: [
            {
              text: [
                paymentDetails.billingAddress?.line1,
                paymentDetails.billingAddress?.town +
                ', ' +
                region +
                paymentDetails.billingAddress?.country?.isocode,
                paymentDetails.billingAddress?.postalCode,
              ],
            },
          ],
          actions: myAccountActions
        };
      })
    );
  }


  getBackToPaymentMethodsEmitter(): EventEmitter<any> {
    return this._backToPaymentMethods;
  }

  setBackToPaymentMethods(value: EventEmitter<any>) {
    this._backToPaymentMethods = value;
  }

  getPaymentMethodCardBraintreeReviewOrder(paymentDetails: BraintreePaymentDetails): Observable<Card> {
    return combineLatest([
      this.translation.translate('paymentForm.payment'),
      this.translation.translate('paymentCard.expires', {
        month: paymentDetails.expiryMonth,
        year: paymentDetails.expiryYear,
      }),
      this.translation.translate('paymentForm.billingAddress'),
    ]).pipe(
      map(([textTitle, textExpires, billingAddress]) => {
        const region = paymentDetails.billingAddress?.region?.isocode
          ? paymentDetails.billingAddress?.region?.isocode + ', '
          : '';
        return {
          title: textTitle,
          textBold: paymentDetails.subscriptionId,
          text: this.configurePaymentMethodCardText(paymentDetails, textExpires),
          paragraphs: [
            {
              text: [
                paymentDetails.billingAddress?.line1,
                paymentDetails.billingAddress?.town +
                ', ' +
                region +
                paymentDetails.billingAddress?.country?.name,
                paymentDetails.billingAddress?.postalCode,
              ],
            },
          ],
        };
      })
    );
  }

  configurePaymentMethodCardText(paymentDetails: BraintreePaymentDetails, textExpires: string): string[]{
    if (paymentDetails.subscriptionId === 'AndroidPayCard'){
      return [paymentText.GOOGLEPAY_ACCOUNT, paymentDetails.payer, paymentDetails.cardNumber, textExpires];
    }else if (paymentDetails.subscriptionId === 'PayPalAccount' || paymentDetails.subscriptionId === 'BrainTreePayPalExpress'){
      return [paymentDetails.subscriptionId, paymentDetails.payer];
    } else if (paymentDetails.subscriptionId === 'VenmoAccount'){
      return [paymentText.VENMO_ACCOUNT, paymentDetails.payer];
    }else if (paymentDetails.subscriptionId === 'CreditCard'){
      return [paymentDetails.payer, paymentDetails.cardType.code, paymentDetails.cardNumber];
    }else if (paymentDetails.subscriptionId === 'ApplePayCard'){
      return [paymentText.APPLEPAY_ACCOUNT, paymentDetails.payer];
    }
      return [paymentDetails.subscriptionId, paymentDetails.cardNumber];
  }

  configurePaymentMethodMyAccountCardText(paymentDetails: PaymentDetails, textExpires: string): string[]{
    if (paymentDetails.subscriptionId === 'AndroidPayCard'){
      return [paymentText.GOOGLEPAY_ACCOUNT, paymentDetails.billingAddress?.email, paymentDetails.cardNumber, textExpires];
    }else if (paymentDetails.subscriptionId === 'PayPalAccount' || paymentDetails.subscriptionId === 'BrainTreePayPalExpress'){
      return [paymentDetails.subscriptionId, paymentDetails.billingAddress?.email];
    } else if (paymentDetails.subscriptionId === 'VenmoAccount'){
      return [paymentText.VENMO_ACCOUNT, paymentDetails.billingAddress?.email];
    }else if (paymentDetails.subscriptionId === 'CreditCard'){
      return [paymentDetails.billingAddress?.email, paymentDetails.cardType.code, paymentDetails.cardNumber];
    }else if (paymentDetails.subscriptionId === 'ApplePayCard'){
      return [paymentText.APPLEPAY_ACCOUNT, paymentDetails.billingAddress?.email];
    }
      return [paymentDetails.subscriptionId, paymentDetails.cardNumber];
  }

}
