import {Component} from '@angular/core';
import {Card, PaymentMethodsComponent} from '@spartacus/storefront';
import {Occ, TranslationService, UserPaymentService} from '@spartacus/core';
import {Observable} from 'rxjs';
import PaymentDetails = Occ.PaymentDetails;
import {
  BraintreePaymentDetailsService,
  BraintreePaymentMethodsUtilsService,
  CheckoutData
} from 'braintree-spartacus-core';

@Component({
  selector: 'bt-payment-methods-myaccount',
  templateUrl: './braintree-payment-methods-myaccount.component.html'
})
export class BraintreePaymentMethodsMyaccountComponent extends PaymentMethodsComponent{

  showAddPaymentInfoForm = false;
  showPaymentInfo = true;
  loadedCheckoutData$: Observable<CheckoutData>;

  constructor(userPaymentService: UserPaymentService,
              translation: TranslationService,
              protected braintreePaymentMethodsUtilsService: BraintreePaymentMethodsUtilsService,
              protected braintreePaymentDetailsService: BraintreePaymentDetailsService) {
    super(userPaymentService, translation);

    this.loadedCheckoutData$ = this.braintreePaymentDetailsService.loadPaymentDetailsForBillingPage('full');
  }

  addPaymentInfoButtonHandle(): void{
    this.showAddPaymentInfoForm = true;
    this.showPaymentInfo = false;
  }


  hideAddPaymentMethod(): void {
    this.showAddPaymentInfoForm = false;
    this.showPaymentInfo = true;
  }

  checkAddPaymentDetailsButtonAvailable(checkoutData: CheckoutData): boolean{
   return (checkoutData.configurationData.storeInVault === 'true' && !(checkoutData.configurationData.intent === 'order'))
          && (checkoutData.payPalPaymentMethod.payPalStandardEnabled
          || Boolean(checkoutData.googlePayPaymentMethod.googlePayEnabled)
          || (!checkoutData.creditCardPaymentMethod.secure3d.valueOf()
          && checkoutData.creditCardPaymentMethod.hostedFieldsEnable)
          || checkoutData.venmoPaymentMethod.venmoEnabled);
  }

  getPaymentMethodCardBraintree(paymentDetails: PaymentDetails): Observable<Card> {
    return this.braintreePaymentMethodsUtilsService.getPaymentMethodCardBraintreeMyAccount(paymentDetails);
  }

  deletePaymentMethod(paymentMethod: PaymentDetails): void{
    super.deletePaymentMethod(paymentMethod);
  }

}
