import {Component, OnInit} from '@angular/core';
import {CheckoutReplenishmentFormService, ScheduleReplenishmentOrderComponent} from '@spartacus/storefront';
import {CheckoutService} from '@spartacus/core';
import {BraintreePaymentDetailsService, CheckoutData, PayPalConfiguration} from 'braintree-spartacus-core';
import {map} from 'rxjs/operators';
import {Observable} from 'rxjs';

@Component({
  selector: 'bt-braintree-replenishment-order',
  templateUrl: './braintree-replenishment-order.component.html',
})
export class BraintreeReplenishmentOrderComponent extends ScheduleReplenishmentOrderComponent implements OnInit {

    payPalConfiguration$: Observable<PayPalConfiguration>;

  constructor(
      protected checkoutService: CheckoutService,
      protected checkoutReplenishmentFormService: CheckoutReplenishmentFormService,
      protected braintreePaymentDetailsService: BraintreePaymentDetailsService
  ) { super(checkoutService, checkoutReplenishmentFormService); }

  ngOnInit(): void {
      super.ngOnInit();
      this.payPalConfiguration$ = this.braintreePaymentDetailsService
        .loadPaymentDetails('configData')
        .pipe(
            map((checkoutData) =>
            {
              return checkoutData.configurationData;
            })
        );
  }

  isReplenishmentAvailable(payPalConfiguration: PayPalConfiguration): boolean{
      return payPalConfiguration.storeInVault === 'true' && payPalConfiguration.intent.toLowerCase() !== 'order';
  }

}
