package com.braintree.servicelayer.i18n.impl;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.c2l.CountryModel;
import de.hybris.platform.servicelayer.exceptions.UnknownIdentifierException;
import de.hybris.platform.servicelayer.i18n.CommonI18NService;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

@UnitTest
public class DefaultBraintreeRegionI18NServiceTest {

    @Mock
    private CommonI18NService commonI18NService;

    @InjectMocks
    private DefaultBraintreeRegionI18NService unit;

    private static final String REGION_SIGN = "za";
    private static final String COUNTY_ISO_CODE = "ua";

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void findRegionShouldReturnNullIfCountryModelNull() {
        Assert.assertNull(unit.findRegion(null, REGION_SIGN));
    }

    @Test
    public void findRegionShouldUseMethodCommonI18ServiceGetRegion() {
        CountryModel countryModel = Mockito.mock(CountryModel.class);
        unit.findRegion(countryModel, REGION_SIGN);
        verify(commonI18NService).getRegion(countryModel, REGION_SIGN);
    }

    @Test
    public void testFindRegion() {
        CountryModel countryModel = Mockito.mock(CountryModel.class);

        when(countryModel.getIsocode()).thenReturn(COUNTY_ISO_CODE);
        when(commonI18NService.getRegion(countryModel, REGION_SIGN))
            .thenThrow(new UnknownIdentifierException(REGION_SIGN));
        when(commonI18NService.getRegion(countryModel, COUNTY_ISO_CODE + "-" + REGION_SIGN))
            .thenThrow(new UnknownIdentifierException(REGION_SIGN));
        unit.findRegion(countryModel, REGION_SIGN);
    }
}
