/**
 *
 */
package com.braintree.controllers.handler;

import braintreehac.services.PayPalButtonConfigurationService;
import com.braintree.configuration.BrainTreeConfigurationListener;
import com.braintree.configuration.service.BrainTreeConfigService;
import com.braintree.constants.BraintreeConstants;
import com.braintree.constants.ControllerConstants;
import com.braintree.controllers.BraintreeaddonControllerConstants;
import com.braintree.enums.BraintreePageType;
import com.braintree.facade.impl.BrainTreeCheckoutFacade;
import com.braintree.facade.impl.BrainTreePaymentFacadeImpl;
import com.braintree.facade.impl.BrainTreeUserFacadeImpl;
import com.braintree.hybris.data.BraintreeUserData;
import com.braintree.hybris.data.BraintreeLocalPaymentMethodData;
import com.braintree.hybris.data.PayPalCheckoutData;
import com.braintree.model.PayPalCreditMessageComponentModel;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hybris.platform.addonsupport.interceptors.BeforeViewHandlerAdaptee;
import de.hybris.platform.cms2.exceptions.CMSItemNotFoundException;
import de.hybris.platform.cms2.servicelayer.services.CMSComponentService;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commercefacades.order.data.OrderData;
import de.hybris.platform.core.Registry;
import de.hybris.platform.servicelayer.session.SessionService;
import de.hybris.platform.util.config.ConfigIntf;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.ModelMap;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static com.braintree.constants.BraintreeConstants.PAYPAL_INTENT_ORDER;
import static com.braintree.constants.BraintreeaddonWebConstants.CREDIT_MESSAGE_COMPONENT;
import static com.braintree.constants.BraintreeaddonWebConstants.IS_CREDIT_MESSAGE_ENABLED;
import static com.braintree.constants.BraintreeaddonWebConstants.PAYMENT_INFOS;
import static com.braintree.controllers.BraintreeaddonControllerConstants.ADD_PAYMENT_METHOD_SHOW;
import static com.braintree.controllers.BraintreeaddonControllerConstants.CLIENT_TOKEN;
import static com.braintree.controllers.BraintreeaddonControllerConstants.PAY_PAL_CHECKOUT_DATA;
import static com.braintree.controllers.BraintreeaddonControllerConstants.Views.Pages.Checkout.CheckoutConfirmationPage;
import static com.braintree.controllers.BraintreeaddonControllerConstants.Views.Pages.MultiStepCheckout.CheckoutSummaryPage;
import static com.braintree.controllers.BraintreeaddonControllerConstants.Views.Pages.MultiStepCheckout.FallbackPage;
import static com.braintree.controllers.BraintreeaddonControllerConstants.Views.Pages.MultiStepCheckout.SilentOrderPostPage;
import static org.apache.commons.lang.StringUtils.isNotBlank;


public class BrainTreeBeforeViewHandler implements BeforeViewHandlerAdaptee {

    private final static Logger LOG = Logger.getLogger(BrainTreeBeforeViewHandler.class);
    private SessionService sessionService;

    private static final String B2C_CHECKOUT_CONFIRMATION_PAGE = "pages/checkout/checkoutConfirmationPage";
    private static final String CART_PAGE = "pages/cart/cartPage";
    private static final String BRAIN_TREE_PAYMENT_DATA = "brainTreePaymentInfoData";
    private static final String ORDER_DATA = "orderData";
    private static final String PAYPAL_BUTTON_CONFIG = "payPalButtonConfig";
    private static final String PAYPAL_MARK_BUTTON_CONFIG = "payPalMarkButtonConfig";
    private static final String B2C_ADD_TO_CART_POPUP_PAGE = "fragments/cart/addToCartPopup";
    private static final String B2C_CART_POPUP_PAGE = "fragments/cart/cartPopup";
    private static final String AccountLayoutPage = "pages/account/accountLayoutPage";
    private static final String B2C_CHECKOUT_CONFIRMATION_LAYOUT_PAGE = "pages/checkout/checkoutConfirmationLayoutPage";
    private static final String B2C_CHECKOUT_DELIVERY_ADDRESS_PAGE = "pages/checkout/multi/addEditDeliveryAddressPage";
    private static final String PAYPAL_CREDIT_MESSAGE = "PayPalCartPageCreditMessageComponent";
    private static final String LPM_IDS = "lpmids";
    private static final String USER_DATA = "userData";


    @Resource(name = "brainTreeCheckoutFacade")
    private BrainTreeCheckoutFacade brainTreeCheckoutFacade;
    @Resource(name = "brainTreePaymentFacadeImpl")
    private BrainTreePaymentFacadeImpl brainTreePaymentFacade;
    @Resource(name = "brainTreeUserFacade")
    private BrainTreeUserFacadeImpl brainTreeUserFacade;
    @Resource(name = "brainTreeConfigService")
    private BrainTreeConfigService brainTreeConfigService;
    @Resource(name = "cmsComponentService")
    private CMSComponentService cmsComponentService;
    @Autowired
    private PayPalButtonConfigurationService payPalButtonConfigurationService;

    private ConfigIntf.ConfigChangeListener configurationChangeListener;

    @Override
    public String beforeView(final HttpServletRequest request, final HttpServletResponse response, final ModelMap model,
        final String viewName) throws Exception {

        if ((!B2C_CART_POPUP_PAGE.equals(viewName) && !B2C_ADD_TO_CART_POPUP_PAGE.equals(viewName))
            && !isPaymentMethodsAvailable(viewName)) {
            return viewName;
        }
        return handleBrainTreeCheckoutScenario(model, viewName);

    }

    private String handleBrainTreeCheckoutScenario(final ModelMap model, final String viewName)
        throws CMSItemNotFoundException {

        if (configurationChangeListener == null) {
            registerConfigChangeLister();
        }

        if (CART_PAGE.equals(viewName)) {
            fillPaymentMethodsInfo(model, BraintreePageType.CART);
            final PayPalCreditMessageComponentModel creditMessageComponent = cmsComponentService
                .getSimpleCMSComponent(PAYPAL_CREDIT_MESSAGE);
            model.addAttribute(CREDIT_MESSAGE_COMPONENT, creditMessageComponent);
            model.addAttribute(IS_CREDIT_MESSAGE_ENABLED, creditMessageComponent.getEnabled());
            model.addAttribute(PAYPAL_BUTTON_CONFIG, payPalButtonConfigurationService.
                parsePayPalButtonConfigDataIntoJsonString(payPalButtonConfigurationService.
                    getPayPalButtonStyleByPageType(BraintreePageType.CART)));
        } else if (ControllerConstants.Views.Pages.MultiStepCheckout.SilentOrderPostPage.equals(viewName)) {
            fillPaymentMethodsInfo(model, BraintreePageType.MARK);
            model.addAttribute(PAYPAL_MARK_BUTTON_CONFIG, payPalButtonConfigurationService.
                parsePayPalButtonConfigDataIntoJsonString(payPalButtonConfigurationService.
                    getPayPalButtonStyleByPageType(BraintreePageType.MARK)));

            if (brainTreeConfigService.getLocalPaymentsEnabled()) {
                BraintreeUserData userData = brainTreeUserFacade.getUserData();
                model.addAttribute(USER_DATA, userData);
            }

            return SilentOrderPostPage;
        } else if (ControllerConstants.Views.Pages.MultiStepCheckout.AccountLayoutPage.equals(viewName)) {
            // add BRAIN_TREE_PAYMENT_DATA for order view page
            final OrderData orderData = (OrderData) model.get(ORDER_DATA);
            if (orderData != null) {
                final String orderCode = orderData.getCode();
                if (isNotBlank(orderCode)) {
                    model.addAttribute(BRAIN_TREE_PAYMENT_DATA,
                        brainTreePaymentFacade.getBrainTreePaymentInfoData(orderCode));
                    return viewName;
                }
            }

            if (Boolean.FALSE.equals(brainTreeConfigService.getPayPalStandardEnabled())
                && Boolean.FALSE.equals(brainTreeConfigService.getHostedFieldEnabled())) {
                return viewName;
            }

            final PayPalCheckoutData payPalCheckoutData = brainTreeCheckoutFacade.
                getPayPalCheckoutData(BraintreePageType.ACCOUNT);
            boolean addPaymentShow = payPalCheckoutData.getCreditCardPaymentMethod().getSecure3d()
                && payPalCheckoutData.getConfigurationData().getIntent().equals(PAYPAL_INTENT_ORDER);
            model.addAttribute(ADD_PAYMENT_METHOD_SHOW, addPaymentShow);
            model.addAttribute(PAY_PAL_CHECKOUT_DATA, payPalCheckoutData);
            model.addAttribute(PAYMENT_INFOS, brainTreeUserFacade.getBrainTreeCCPaymentInfos(true));
            return AccountLayoutPage;
        } else if (B2C_CHECKOUT_CONFIRMATION_LAYOUT_PAGE.equals(viewName)) {
            final String orderCode = ((OrderData) model.get(ORDER_DATA)).getCode();
            if (isNotBlank(orderCode)) {
                model.addAttribute(BRAIN_TREE_PAYMENT_DATA,
                    brainTreePaymentFacade.getBrainTreePaymentInfoData(orderCode));
            }
            return viewName;
        } else if (ControllerConstants.Views.Pages.MultiStepCheckout.CheckoutSummaryPage.equals(viewName)) {
            model.addAttribute(BRAIN_TREE_PAYMENT_DATA, brainTreePaymentFacade.getBrainTreePaymentInfoData());
            return CheckoutSummaryPage;
        } else if (B2C_CHECKOUT_CONFIRMATION_PAGE.equals(viewName)) {
            final String orderCode = ((OrderData) model.get(ORDER_DATA)).getCode();
            if (isNotBlank(orderCode)) {
                model.addAttribute(BRAIN_TREE_PAYMENT_DATA,
                    brainTreePaymentFacade.getBrainTreePaymentInfoData(orderCode));
            }
            return CheckoutConfirmationPage;
        } else if (B2C_ADD_TO_CART_POPUP_PAGE.equals(viewName)) {
            if (isPaymentMethodsAvailable(viewName)) {
                fillPaymentMethodsInfo(model, BraintreePageType.MINICART);
                model.addAttribute(PAYPAL_BUTTON_CONFIG, payPalButtonConfigurationService.
                    parsePayPalButtonConfigDataIntoJsonString(payPalButtonConfigurationService.
                        getPayPalButtonStyleByPageType(BraintreePageType.MINICART)));
                return BraintreeaddonControllerConstants.Views.Fragments.Cart.AddToCartPopup;
            }
            return viewName;
        } else if (B2C_CART_POPUP_PAGE.equals(viewName)) {
            if (isPaymentMethodsAvailable(viewName)) {
                fillPaymentMethodsInfo(model, BraintreePageType.MINICART);
                model.addAttribute(PAYPAL_BUTTON_CONFIG, payPalButtonConfigurationService.
                    parsePayPalButtonConfigDataIntoJsonString(payPalButtonConfigurationService.
                        getPayPalButtonStyleByPageType(BraintreePageType.MINICART)));
                return BraintreeaddonControllerConstants.Views.Fragments.Cart.CartPopup;
            }
            return viewName;
        } else if (B2C_CHECKOUT_DELIVERY_ADDRESS_PAGE.equals(viewName)
            && getSessionService().getAttribute("braintree.general.error.shippingAddress") != null) {
            model.addAttribute("accErrorMsgs",
                getSessionService().getAttribute("braintree.general.error.shippingAddress"));
            getSessionService().removeAttribute("braintree.general.error.shippingAddress");
        } else if (ControllerConstants.Views.Pages.MultiStepCheckout.FallbackPage.equals(viewName)) {
            model.addAttribute(CLIENT_TOKEN, brainTreeCheckoutFacade.generateClientToken());
            return FallbackPage;
        }
        return viewName;
    }

    private void fillPaymentMethodsInfo(final ModelMap model, BraintreePageType pageType) {
        ObjectMapper om = new ObjectMapper();
        final PayPalCheckoutData payPalCheckoutData = brainTreeCheckoutFacade.getPayPalCheckoutData(pageType);
        model.addAttribute(PAY_PAL_CHECKOUT_DATA, payPalCheckoutData);
        List<CCPaymentInfoData> brainTreeCCPaymentInfos = brainTreeUserFacade.getBrainTreeCCPaymentInfos(true);
        brainTreeCCPaymentInfos.forEach(brainTreePaymentFacade::setPaymentMethodNonce);
        model.addAttribute("payPalShouldBeSaved", !isPayPalAccountSaved(brainTreeCCPaymentInfos));
        model.addAttribute(PAYMENT_INFOS, brainTreePaymentFacade.getAvailablePayments(brainTreeCCPaymentInfos));
        if (pageType.equals(BraintreePageType.MARK)) {
            List<String> lpmids = new ArrayList<>();
            for (BraintreeLocalPaymentMethodData lpm :
                payPalCheckoutData.getLocalPaymentMethod().getLocalPaymentMethods()) {
                lpmids.add(lpm.getCode());
            }
            try {
                model.addAttribute(LPM_IDS, om.writeValueAsString(lpmids));
            } catch (IOException e) {
                LOG.error("Error during write value");
            }
        }
    }

    private boolean isPayPalAccountSaved(List<CCPaymentInfoData> savedPayments) {
        for (CCPaymentInfoData payment : savedPayments) {
            if (BraintreeConstants.PAYPAL_PAYMENT.equals(payment.getSubscriptionId())
                || BraintreeConstants.PAY_PAL_EXPRESS_CHECKOUT
                .equals(payment.getSubscriptionId())) {
                return true;
            }
        }
        return false;
    }

    private boolean isPaymentMethodsAvailable(String viewName) {
        if (B2C_CART_POPUP_PAGE.equals(viewName) || B2C_ADD_TO_CART_POPUP_PAGE.equals(viewName)) {
            return brainTreeConfigService.getPayPalExpressEnabled() || brainTreeConfigService.getApplePayEnabled()
                || brainTreeConfigService.getGooglePayEnabled();
        } else if (AccountLayoutPage.equals(viewName)) {
            return brainTreeConfigService.getPayPalStandardEnabled() || brainTreeConfigService.getHostedFieldEnabled()
                || brainTreeConfigService.getVenmoEnabled();
        } else {
            return brainTreeConfigService.isOneOfPaymentMethodsEnabled();
        }
    }

    private void registerConfigChangeLister() {
        final ConfigIntf config = Registry.getMasterTenant().getConfig();
        configurationChangeListener = new BrainTreeConfigurationListener();
        config.registerConfigChangeListener(configurationChangeListener);
    }

    public SessionService getSessionService() {
        return sessionService;
    }

    public void setSessionService(SessionService sessionService) {
        this.sessionService = sessionService;
    }
}
