package com.braintree.payment.validators;

import com.braintree.exceptions.ResourceErrorMessage;
import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

@UnitTest
public class EditPaymentMethodValidatorTest {

    private EditPaymentMethodValidator unit;

    private static final String CARD_HOLDER = "cardHolder";
    private static final String CORRECT_EXPIRATION_DATE = "02/21";
    private static final String INCORRECT_EXPIRATION_DATE = "02/21/12";
    private static final String CORRECT_CVV = "cvv";
    private static final String INCORRECT_CVV = "cvvvvv";

    private static final String INVALID_CVV_MESSAGE_CODE =
        "text.account.profile.paymentCart.editPaymentMethod.invalid.cvv";

    private static final String INVALID_EXPIRATION_DATE_MESSAGE_CODE =
        "text.account.profile.paymentCart.editPaymentMethod.invalid.expirationDate";

    private static final String ABSENCE_EXPIRATION_DATE_MESSAGE_CODE =
        "text.account.profile.paymentCart.editPaymentMethod.required.expirationDate";

    @Before
    public void setUp() throws Exception {
        unit = new EditPaymentMethodValidator();
    }


    @Test()
    public void testValidatorWithIncorrectCVV() {
        ResourceErrorMessage resourceErrorMessage = unit.validate(CARD_HOLDER, CORRECT_EXPIRATION_DATE, INCORRECT_CVV);
        Assert.assertEquals(INVALID_CVV_MESSAGE_CODE, resourceErrorMessage.getMessageKey());
    }

    @Test
    public void testValidatorWithIncorrectExpirationDate() {
        ResourceErrorMessage resourceErrorMessage = unit.validate(CARD_HOLDER, INCORRECT_EXPIRATION_DATE, CORRECT_CVV);
        Assert.assertEquals(INVALID_EXPIRATION_DATE_MESSAGE_CODE, resourceErrorMessage.getMessageKey());
    }

    @Test
    public void testValidatorWithWithoutExpirationDate() {
        ResourceErrorMessage resourceErrorMessage = unit.validate(CARD_HOLDER, "", CORRECT_CVV);
        Assert.assertEquals(ABSENCE_EXPIRATION_DATE_MESSAGE_CODE, resourceErrorMessage.getMessageKey());
    }

    @Test
    public void testValidatorWithIncorrectExpirationDateLength() {
        ResourceErrorMessage resourceErrorMessage = unit.validate(CARD_HOLDER, CORRECT_EXPIRATION_DATE, CORRECT_CVV);
        Assert.assertEquals(INVALID_EXPIRATION_DATE_MESSAGE_CODE, resourceErrorMessage.getMessageKey());
    }
}
