/**
 *
 */
package com.braintree.controllers.pages;

import static com.braintree.controllers.Braintreeb2baddonControllerConstants.PAY_PAL_ADDRESS_ERROR;
import static com.braintree.controllers.Braintreeb2baddonControllerConstants.PAY_PAL_HAED_ERROR;
import static com.braintree.controllers.Braintreeb2baddonControllerConstants.Views.Pages.MultiStepCheckout.CheckoutOrderPageErrorPage;
import static de.hybris.platform.util.localization.Localization.getLocalizedString;

import de.hybris.platform.acceleratorstorefrontcommons.annotations.RequireHardLogIn;
import de.hybris.platform.acceleratorstorefrontcommons.constants.WebConstants;
import de.hybris.platform.acceleratorstorefrontcommons.controllers.util.GlobalMessages;
import de.hybris.platform.cms2.exceptions.CMSItemNotFoundException;
import de.hybris.platform.commercefacades.user.UserFacade;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.commerceservices.order.CommerceCartModificationException;
import de.hybris.platform.order.CartService;
import de.hybris.platform.payment.AdapterException;

import java.io.IOException;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import com.fasterxml.jackson.core.JsonGenerationException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.braintree.constants.BraintreeConstants;
import com.braintree.controllers.handler.PayPalResponseExpressCheckoutHandler;
import com.braintree.controllers.handler.PayPalUserLoginHandler;
import com.braintree.facade.BrainTreeB2BCheckoutFacade;
import com.braintree.facade.impl.BrainTreeCheckoutFacade;
import com.braintree.facade.impl.BrainTreePaymentFacadeImpl;
import com.braintree.hybris.data.BrainTreeSubscriptionInfoData;
import com.braintree.hybris.data.PayPalAddressData;
import com.braintree.hybris.data.PayPalCheckoutData;
import com.braintree.hybris.data.PayPalExpressResponse;
import com.braintree.hybris.data.PayPalMiniCartResponse;


@Controller
@RequireHardLogIn
@RequestMapping(value = "/braintree/paypal/checkout")
public class PayPalPaymentController extends BrainTreeSummaryCheckoutStepController {

    private static final Logger LOG = Logger.getLogger(PayPalPaymentController.class);
    private static final String REDIRECT_TO_PAYMENT_INFO_PAGE = REDIRECT_PREFIX + "/my-account/payment-details";
    private static final String SAVE_PAYMENT_INFO = "isSaved";
    private static final String DEVICE_DATA = "device_data";

    @Resource(name = "payPalUserLoginHandler")
    PayPalUserLoginHandler payPalUserLoginHandler;

    @Resource(name = "payPalResponseExpressCheckoutHandler")
    PayPalResponseExpressCheckoutHandler payPalResponseExpressCheckoutHandler;

    @Resource(name = "brainTreePaymentFacadeImpl")
    private BrainTreePaymentFacadeImpl brainTreePaymentFacade;

    @Resource(name = "userFacade")
    private UserFacade userFacade;

    @Resource(name = "brainTreeCheckoutFacade")
    private BrainTreeCheckoutFacade brainTreeCheckoutFacade;

    @Resource(name = "brainTreeB2BCheckoutFacade")
    private BrainTreeB2BCheckoutFacade brainTreeB2BCheckoutFacade;

    @Resource
    private CartService cartService;

    @RequestMapping(value = "/express", method = RequestMethod.POST)
    public String doHandleHopResponse(final Model model, final RedirectAttributes redirectAttributes,
        final HttpServletRequest request, final HttpServletResponse response) throws CMSItemNotFoundException,
        CommerceCartModificationException {

        PayPalExpressResponse payPalExpressResponse = null;
        boolean isSavePaymentInfo = false;
        String deviceData = null;
        try {
            payPalExpressResponse = payPalResponseExpressCheckoutHandler.handlePayPalResponse(request);
            isSavePaymentInfo = Boolean.parseBoolean(request.getParameter(SAVE_PAYMENT_INFO));
            deviceData = request.getParameter(DEVICE_DATA);
        } catch (final IllegalArgumentException exeption) {
            handleErrors(exeption.getMessage(), model);
            return CheckoutOrderPageErrorPage;
        }

        String paymentProvider = BraintreeConstants.PAY_PAL_EXPRESS_CHECKOUT;
        final String payerEmail = payPalExpressResponse.getDetails().getEmail();

        if (payPalExpressResponse.getType().equals(BraintreeConstants.APPLE_PAY_CARD)) {
            paymentProvider = BraintreeConstants.APPLE_PAY_CARD;
        } else if (payPalExpressResponse.getType().equals(BraintreeConstants.ANDROID_PAY_CARD)) {
            paymentProvider = BraintreeConstants.ANDROID_PAY_CARD;
        }

        final BrainTreeSubscriptionInfoData subscriptionInfo = buildSubscriptionInfo(payPalExpressResponse.getNonce(),
            paymentProvider, isSavePaymentInfo, Boolean.FALSE, payerEmail);

        final PayPalAddressData payPalShippingAddress = payPalExpressResponse.getDetails().getShippingAddress();
        AddressData hybrisShippingAddress = null;
        if (payPalShippingAddress != null && brainTreeCheckoutFacade.getCheckoutCart().getDeliveryAddress() == null) {
            hybrisShippingAddress = payPalResponseExpressCheckoutHandler
                .getPayPalAddress(payPalExpressResponse.getDetails(),
                    payPalShippingAddress);

            userFacade.addAddress(hybrisShippingAddress);
            brainTreeCheckoutFacade.setDeliveryAddress(hybrisShippingAddress);
        } else {
            if (brainTreeCheckoutFacade.getCheckoutCart().getDeliveryAddress() == null) {
                LOG.error("Shipping address from pay pal is empty!");
                final String errorMessage = getLocalizedString(PAY_PAL_ADDRESS_ERROR);
                handleErrors(errorMessage, model);
                return CheckoutOrderPageErrorPage;
            }
        }

        if (deviceData != null) {
            subscriptionInfo.setDeviceData(deviceData);
        }

        final PayPalAddressData payPalBillingAddress = payPalExpressResponse.getDetails().getBillingAddress();
        if (payPalBillingAddress != null) {
            final AddressData hybrisBillingAddress = payPalResponseExpressCheckoutHandler.getPayPalAddress(
                payPalExpressResponse.getDetails(), payPalBillingAddress);
            subscriptionInfo.setAddressData(hybrisBillingAddress);
        } else {
            LOG.warn("No billing address provide by Pay Pal. Use billing address as shipping...");
            subscriptionInfo.setAddressData(hybrisShippingAddress);
        }

        if (cartService.getSessionCart().getDeliveryMode() == null) {
            brainTreeCheckoutFacade.setCheapestDeliveryModeForCheckout();
            LOG.info("Set default cheapest delivery mode for PayPal simple flow");
            if (cartService.getSessionCart().getDeliveryMode() == null) {
                LOG.info("Delivery address provided by Pay Pal is not supported.");
                brainTreeCheckoutFacade.setDeliveryAddress(null);
            }
        }

        try {
            brainTreePaymentFacade.completeCreateSubscription(subscriptionInfo);
            brainTreeB2BCheckoutFacade.setCardPaymentType();
        } catch (final Exception exception) {
            final String errorMessage = getLocalizedString("braintree.billing.general.error");
            handleErrors(errorMessage, model);
            return CheckoutOrderPageErrorPage;
        }

        return REDIRECT_PREFIX + "/checkout/multi/summary/view";
    }

    @RequestMapping(value = "/add-payment-method", method = RequestMethod.POST)
    public String addPaymentMethod(final Model model, final RedirectAttributes redirectAttributes,
        @RequestParam(value = "selectedAddressCode", required = false) final String selectedAddressCode,
        final HttpServletRequest request, final HttpServletResponse response) throws CMSItemNotFoundException {
        PayPalExpressResponse payPalExpressResponse = null;
        AddressData hybrisBillingAddress = null;
        try {
            payPalExpressResponse = payPalResponseExpressCheckoutHandler.handlePayPalResponse(request);
        } catch (final IllegalArgumentException exeption) {
            handleErrors(exeption.getMessage(), model);
            return CheckoutOrderPageErrorPage;
        }

        String payerEmail = payPalExpressResponse.getDetails().getEmail();
        String paymentProvider = BraintreeConstants.PAY_PAL_EXPRESS_CHECKOUT;

        if (payPalExpressResponse.getType().equals(BraintreeConstants.APPLE_PAY_CARD)) {
            paymentProvider = BraintreeConstants.APPLE_PAY_CARD;
        } else if (payPalExpressResponse.getType().equals(BraintreeConstants.VENMO_CHECKOUT)) {
            paymentProvider = BraintreeConstants.VENMO_CHECKOUT;
            hybrisBillingAddress = userFacade.getAddressForCode(selectedAddressCode);
        } else if (payPalExpressResponse.getType().equals(BraintreeConstants.ANDROID_PAY_CARD)) {
            paymentProvider = BraintreeConstants.ANDROID_PAY_CARD;
        }

        final BrainTreeSubscriptionInfoData subscriptionInfo = buildSubscriptionInfo(payPalExpressResponse.getNonce(),
            paymentProvider, Boolean.FALSE, Boolean.TRUE, payerEmail);

        final PayPalAddressData payPalBillingAddress = payPalExpressResponse.getDetails().getBillingAddress();
        if (payPalBillingAddress != null) {
            hybrisBillingAddress = payPalResponseExpressCheckoutHandler.getPayPalAddress(
                payPalExpressResponse.getDetails(), payPalBillingAddress);
            hybrisBillingAddress.setEmail(payPalExpressResponse.getDetails().getEmail());
            subscriptionInfo.setAddressData(hybrisBillingAddress);
        } else if (paymentProvider.equals(BraintreeConstants.VENMO_CHECKOUT)) {
            subscriptionInfo.setEmail(payPalExpressResponse.getDetails().getUsername());
            subscriptionInfo.setAddressData(hybrisBillingAddress);
        } else {
            LOG.warn("No billing address provide by Pay Pal. Use empty billing address...");
            hybrisBillingAddress = new AddressData();
            subscriptionInfo.setAddressData(hybrisBillingAddress);
        }

        try {
            brainTreePaymentFacade.completeCreateSubscription(subscriptionInfo, false);
        } catch (final Exception exception) {
            final String errorMessage = getLocalizedString("braintree.billing.general.error");
            handleErrors(errorMessage, model);
            return CheckoutOrderPageErrorPage;
        }
        GlobalMessages.addFlashMessage(redirectAttributes, GlobalMessages.CONF_MESSAGES_HOLDER,
            getLocalizedString("text.account.profile.paymentCart.addPaymentMethod.success"));
        return REDIRECT_TO_PAYMENT_INFO_PAGE;
    }

    @ResponseBody
    @RequestMapping(value = "/shippingAddressError", method = RequestMethod.POST)
    public void handleShippingAddressError(final Model model,
        final @RequestParam(value = "errorMessage", required = false) String errorMessage)
        throws CMSItemNotFoundException {
        LOG.error("Not correct shipping address. Error message: " + errorMessage);
        GlobalMessages
            .addMessage(model, "accErrorMsgs", "braintree.general.error.shippingAddress", new String[]{errorMessage});
        getSessionService().getCurrentSession().setAttribute("braintree.general.error.shippingAddress",
            model.asMap().get("accErrorMsgs"));
    }

    @RequestMapping(value = "/mini/express", method = RequestMethod.GET)
    @RequireHardLogIn
    @ResponseBody
    public String doInitializeMiniCartPaypalShortcut() throws CMSItemNotFoundException, JsonGenerationException,
        JsonMappingException, IOException {
        final String jsonInString = buildPayPalMiniCartResponse();
        return jsonInString;
    }

    private String buildPayPalMiniCartResponse() throws JsonGenerationException, JsonMappingException, IOException {
        final ObjectMapper mapper = new ObjectMapper();
        final PayPalMiniCartResponse payPalMiniCartResponse = new PayPalMiniCartResponse();
        final PayPalCheckoutData payPalCheckoutData = brainTreeCheckoutFacade.generateConfigurationData();
        payPalMiniCartResponse.setCheckoutData(payPalCheckoutData);
        return mapper.writeValueAsString(payPalMiniCartResponse);
    }

    private BrainTreeSubscriptionInfoData buildSubscriptionInfo(final String nonce, final String paymentProvider,
        boolean shouldBeSaved, boolean isSaved, final String email) {
        final BrainTreeSubscriptionInfoData subscriptionInfo = new BrainTreeSubscriptionInfoData();
        subscriptionInfo.setPaymentProvider(paymentProvider);
        subscriptionInfo.setShouldBeSaved(shouldBeSaved);
        subscriptionInfo.setSavePaymentInfo(isSaved);
        subscriptionInfo.setNonce(nonce);
        subscriptionInfo.setEmail(email);

        return subscriptionInfo;
    }

    private void handleErrors(final String errorsDetail, final Model model) throws CMSItemNotFoundException {
        model.addAttribute("errorsDetail", errorsDetail);
        final String redirectUrl = REDIRECT_URL_CART;
        model.addAttribute("redirectUrl", redirectUrl.replace(REDIRECT_PREFIX, ""));
        model.addAttribute(WebConstants.BREADCRUMBS_KEY,
            getResourceBreadcrumbBuilder().getBreadcrumbs("checkout.multi.hostedOrderPageError.breadcrumb"));
        storeCmsPageInModel(model, getContentPageForLabelOrId(MULTI_CHECKOUT_SUMMARY_CMS_PAGE_LABEL));
        setUpMetaDataForContentPage(model, getContentPageForLabelOrId(MULTI_CHECKOUT_SUMMARY_CMS_PAGE_LABEL));

        GlobalMessages.addErrorMessage(model, getLocalizedString(PAY_PAL_HAED_ERROR));
    }

}
