package com.braintree.facade.backoffice.impl;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import com.braintree.model.BrainTreePaymentInfoModel;
import com.braintree.transaction.service.BrainTreeTransactionService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.AbstractOrderModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.ordermanagementfacades.payment.data.PaymentTransactionEntryData;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import de.hybris.platform.servicelayer.model.ModelService;
import java.math.BigDecimal;
import java.util.Map;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

@UnitTest
public class BraintreeBackofficeAuthorizeFacadeImplTest {

    @Mock
    private BrainTreeTransactionService brainTreeTransactionService;

    @Mock
    private Converter<PaymentTransactionEntryModel, PaymentTransactionEntryData> ordermanagementPaymentTransactionEntryConverter;

    @Mock
    private ModelService modelService;

    @InjectMocks
    private BraintreeBackofficeAuthorizeFacadeImpl unit;

    private static final String PAYMENT_PROVIDER = "paymentProvider";
    private static final BigDecimal AMOUNT = BigDecimal.TEN;
    private static final Double TOTAL_PRICE = 200d;

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void isAuthorizationPossobleShouldReturnFalseIfOrderModelNull() {
        Assert.assertFalse(unit.isAuthorizePossible(null));
    }

    @Test
    public void authorizationPaymentShouldUseTransactionServiceForCreatingAuthorizationTransaction() {
        AbstractOrderModel orderModel = Mockito.mock(AbstractOrderModel.class);
        Map<String, String> customFields = Mockito.mock(Map.class);
        unit.authorizePayment(orderModel, customFields, BigDecimal.TEN);
        verify(brainTreeTransactionService).createAuthorizationTransaction(orderModel, customFields, AMOUNT);
    }

    @Test
    public void isAuthorizePossibleShouldCheckOrderOnIntent() {
        OrderModel orderModel = Mockito.mock(OrderModel.class);
        BrainTreePaymentInfoModel paymentInfoModel = Mockito.mock(BrainTreePaymentInfoModel.class);

        when(paymentInfoModel.getPaymentProvider()).thenReturn(PAYMENT_PROVIDER);
        when(orderModel.getPaymentInfo()).thenReturn(paymentInfoModel);
        when(orderModel.getTotalPrice()).thenReturn(TOTAL_PRICE);

        unit.isAuthorizePossible(orderModel);
        verify(paymentInfoModel).getPayPalIntent();
    }
}
