package com.braintree.facade.backoffice.impl;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import com.braintree.configuration.service.BrainTreeConfigService;
import com.braintree.model.BrainTreePaymentInfoModel;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.model.ModelService;
import java.util.ArrayList;
import java.util.List;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

@UnitTest
public class BraintreeBackofficeMultiCaptureFacadeImplTest {

    @Mock
    private ModelService modelService;

    @Mock
    private BrainTreeConfigService brainTreeConfigService;

    @InjectMocks
    private BraintreeBackofficeMultiCaptureFacadeImpl unit;

    private static final String PAYPAL_INTENT = "paypalIntent";
    private static final String PAYMENT_PROVIDER_PROPERTY = "paymentProvider";

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void isMultiCapturePossibleShouldReturnFalseIfOrderModelNull() {
        Assert.assertFalse(unit.isMultiCapturePossible(null));
    }

    @Test
    public void isMultiCapturePossibleShouldReturnFalseIfPaymentInfoNotBraintreeType() {
        OrderModel orderModel = Mockito.mock(OrderModel.class);
        Assert.assertFalse(unit.isMultiCapturePossible(orderModel));
    }

    @Test
    public void isMultiCapturePossibleShouldReturnFalseWhenOrderNotHaveTransactions() {
        OrderModel orderModel = Mockito.mock(OrderModel.class);
        BrainTreePaymentInfoModel paymentInfoModel = Mockito.mock(BrainTreePaymentInfoModel.class);

        when(paymentInfoModel.getPayPalIntent()).thenReturn(PAYPAL_INTENT);
        when(paymentInfoModel.getProperty(PAYMENT_PROVIDER_PROPERTY)).thenReturn(PAYMENT_PROVIDER_PROPERTY);
        when(orderModel.getPaymentInfo()).thenReturn(paymentInfoModel);
        Assert.assertFalse(unit.isMultiCapturePossible(orderModel));
    }

    @Test
    public void getMultiCaptureableTransactionsShouldGetTypeAndStatusFromTransactionEntry() {
        OrderModel orderModel = Mockito.mock(OrderModel.class);
        PaymentTransactionModel transactionModel = Mockito.mock(PaymentTransactionModel.class);
        List<PaymentTransactionModel> transactionModelList = new ArrayList<>();
        transactionModelList.add(transactionModel);

        PaymentTransactionEntryModel entryModel = Mockito.mock(PaymentTransactionEntryModel.class);
        List<PaymentTransactionEntryModel> entryModels = new ArrayList<>();
        entryModels.add(entryModel);

        when(entryModel.getType()).thenReturn(PaymentTransactionType.AUTHORIZATION);
        when(entryModel.getTransactionStatus()).thenReturn(TransactionStatus.ACCEPTED.name());
        when(transactionModel.getEntries()).thenReturn(entryModels);
        when(orderModel.getPaymentTransactions()).thenReturn(transactionModelList);

        unit.getMultiCaptureableTransactions(orderModel);
        verify(entryModel).getType();
        verify(entryModel).getTransactionStatus();
    }
}
