package com.braintree.controllers;

import braintreehac.services.PayPalButtonConfigurationService;
import com.braintree.enums.BraintreePageType;
import com.braintree.facade.impl.BrainTreeCheckoutFacade;
import com.braintree.hybris.data.PayPalCheckoutData;
import com.braintree.hybris.data.ws.BraintreeAssetsWsDTO;
import com.braintree.hybris.data.ws.PayPalCheckoutWsDTO;
import com.braintree.hybris.data.ws.PayPalButtonStyleWsDTO;
import com.braintree.validators.BraintreeButtonTypeValidator;
import de.hybris.platform.braintreehac.data.PayPalButtonStyleData;
import de.hybris.platform.webservicescommons.swagger.ApiBaseSiteIdParam;
import de.hybris.platform.webservicescommons.swagger.ApiBaseSiteIdUserIdAndCartIdParam;
import de.hybris.platform.ycommercewebservices.v2.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.GetMapping;

import javax.annotation.Resource;

@Controller
@RequestMapping(value = "/{baseSiteId}")
@Api(tags = "Braintree Payment")
public class BraintreePaymentController extends BaseController {

    @Resource(name = "braintreeButtonTypeValidator")
    private BraintreeButtonTypeValidator buttonTypeValidator;

    @Resource(name = "brainTreeCheckoutFacade")
    private BrainTreeCheckoutFacade brainTreeCheckoutFacade;

    @Resource(name = "payPalButtonConfigurationService")
    private PayPalButtonConfigurationService payPalButtonConfigurationService;

    @GetMapping("/users/{userId}/braintree/payment")
    @ResponseStatus(value = HttpStatus.OK)
    @ResponseBody
    @ApiBaseSiteIdParam
    @ApiOperation(value = " Get the Braintree data for client-side payment initialization",
        notes = "Get a data for all payment methods")
    public PayPalCheckoutWsDTO getPaymentData(
        @ApiParam(value = "Payment method identifier.", allowableValues = "paypal, googlepay, venmo, applepay," +
            " localpayments, creditcard, configData, full", required = true) @RequestParam final String paymentMethod) {
        PayPalCheckoutData payPalCheckoutData = brainTreeCheckoutFacade.getPayPalCheckoutData(BraintreePageType.MARK);
        return getDataMapper().map(payPalCheckoutData, PayPalCheckoutWsDTO.class, paymentMethod);
    }

    @GetMapping("/users/{userId}/braintree/payment/button")
    @ResponseStatus(value = HttpStatus.OK)
    @ResponseBody
    @ApiBaseSiteIdParam
    @ApiOperation(value = " Get the PayPal Smart Button style data.")
    public PayPalButtonStyleWsDTO getButtonStyle(
        @ApiParam(value = "Button type identifier.", allowableValues = "minicart, cart, mark",
            required = true) @RequestParam final String buttonType) {
        validate(buttonType, buttonType, buttonTypeValidator);
        PayPalButtonStyleData payPalButtonStyleData = payPalButtonConfigurationService.
            getPayPalButtonStyleByPageType(BraintreePageType.valueOf(buttonType.toUpperCase()));
        return getDataMapper().map(payPalButtonStyleData, PayPalButtonStyleWsDTO.class);
    }

    @GetMapping("/braintree/payment/assets")
    @ResponseStatus(value = HttpStatus.OK)
    @ResponseBody
    @ApiBaseSiteIdParam
    @ApiOperation(value = " Get assets for checkout.")
    public BraintreeAssetsWsDTO getAssets() {
        return getDataMapper().map(brainTreeCheckoutFacade.getBraintreeAssets(), BraintreeAssetsWsDTO.class);
    }
}
