package com.braintree.validators;

import com.braintree.hybris.data.ws.PayPalDetailsWsDTO;
import org.springframework.util.Assert;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import static com.braintree.constants.BraintreeoccaddonWebConstants.FIELD_REQUIRED_MESSAGE_ID;

public class PayPalDetailsValidator implements BrainTreeValidator {

    private Validator paymentAddressValidator;

    @Override
    public boolean supports(final Class clazz) {
        return PayPalDetailsWsDTO.class.isAssignableFrom(clazz);
    }

    @Override
    public void validate(final Object target, final Errors errors) {

        final PayPalDetailsWsDTO brainTreeDetails = (PayPalDetailsWsDTO) target;

        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "email", FIELD_REQUIRED_MESSAGE_ID);
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "countryCode", FIELD_REQUIRED_MESSAGE_ID);
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "payerId", FIELD_REQUIRED_MESSAGE_ID);
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "phone", FIELD_REQUIRED_MESSAGE_ID);
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "firstName", FIELD_REQUIRED_MESSAGE_ID);
        ValidationUtils.rejectIfEmptyOrWhitespace(errors, "lastName", FIELD_REQUIRED_MESSAGE_ID);

        Assert.notNull(brainTreeDetails.getBillingAddress(), "BillingAddress object must not be null");
        validate(brainTreeDetails.getBillingAddress(), "billingAddress", getPaymentAddressValidator());
    }

    public Validator getPaymentAddressValidator() {
        return paymentAddressValidator;
    }

    public void setPaymentAddressValidator(Validator paymentAddressValidator) {
        this.paymentAddressValidator = paymentAddressValidator;
    }
}
