package com.braintree.converters.populators;

import com.braintree.hybris.data.ws.GooglePayAddressWsDTO;
import com.braintree.paypal.validator.PayPalAddressDetailValidator;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.commercefacades.user.data.CountryData;
import de.hybris.platform.commercefacades.user.data.RegionData;
import de.hybris.platform.converters.Populator;
import de.hybris.platform.servicelayer.dto.converter.ConversionException;

public class GooglePayAddressPopulator implements Populator<GooglePayAddressWsDTO, AddressData> {

    private PayPalAddressDetailValidator payPalAddressDetailValidator;

    @Override
    public void populate(GooglePayAddressWsDTO googlePayAddress, AddressData addressData) throws ConversionException {
        final boolean isCountryIsoCodeValid = getPayPalAddressDetailValidator()
            .validatePayPalCountryCode(googlePayAddress.getCountryCode());
        final boolean isRegionIsoCodeValid = getPayPalAddressDetailValidator()
            .validatePayPalRegionCode(googlePayAddress.getCountryCode(), googlePayAddress.getAdministrativeArea());

        addressData.setBillingAddress(Boolean.TRUE);
        addressData.setLine1(googlePayAddress.getAddress1());
        addressData.setLine2(googlePayAddress.getAddress2());
        addressData.setPhone(googlePayAddress.getPhoneNumber());
        addressData.setPostalCode(googlePayAddress.getPostalCode());
        addressData.setTown(googlePayAddress.getLocality());
        if (isCountryIsoCodeValid) {
            final CountryData country = new CountryData();
            country.setIsocode(googlePayAddress.getCountryCode());
            addressData.setCountry(country);
        }

        if (isRegionIsoCodeValid) {
            final RegionData region = new RegionData();
            region.setIsocode(googlePayAddress.getCountryCode() + "-" + googlePayAddress.getAdministrativeArea());
            addressData.setRegion(region);
        }
        //TODO
        addressData.setFirstName(googlePayAddress.getName());
    }

    public PayPalAddressDetailValidator getPayPalAddressDetailValidator() {
        return payPalAddressDetailValidator;
    }

    public void setPayPalAddressDetailValidator(PayPalAddressDetailValidator payPalAddressDetailValidator) {
        this.payPalAddressDetailValidator = payPalAddressDetailValidator;
    }
}
